import logging
import datetime
import glob
import os
import numpy as np

import iris
import iris.cube
import iris.analysis

import diagonals
import diagonals.ohc as ohc
from diagonals.mesh_helpers.nemo import Nemo

from pycompss.api.task import task
from pycompss.api.parameter import *
from pycompss.api.api import compss_barrier, compss_wait_on

MESH_FILE = "/esarchive/autosubmit/con_files/mesh_mask_nemo.Ec3.2_O1L75.nc"
DATA_FOLDER = "/esarchive/exp/ecearth/a1rh/cmorfiles/CMIP/EC-Earth-Consortium/EC-Earth3/historical/r2i1p1f1/Omon/thetao/gn/v20190219/"
REGIONS_FILE = "/esarchive/autosubmit/con_files/mask.regions.Ec3.2_O1L75.nc"

logger = logging.getLogger(__name__)


def main():
    logging.basicConfig(level=logging.INFO)
    start = datetime.datetime.now()
    logger.info('Starting at %s', start)
    one_layer = ((0, 300),)
    mesh = Nemo(MESH_FILE, REGIONS_FILE)
    multiple_layers = ((0, 200), (200, 700), (700, 2000),
                       (0, 100), (100, 200),(200, 300), (300, 400),
                       (400, 500), (500, 1000),)
    e3t = mesh.get_k_length()
    mask = mesh.get_landsea_mask()
    depth = mesh.get_depth(cell_point='W')
    weights = ohc.get_weights(one_layer, mask, e3t, depth)

    basin_name, basins = load_basins()
    areacello = mesh.get_areacello()
    area = ohc.get_basin_area(areacello, basins)
    filepaths = list(glob.glob(os.path.join(DATA_FOLDER, 'thetao_*.nc')))
    for filepath in filepaths[0:5]:
        filename = os.path.basename(filepath)
        thetao = load_thetao(filepath)
        ohc_2D, ohc_1D = ohc.compute(multiple_layers, weights, thetao, area)
        save_2d(multiple_layers, basin_name, ohc_2D, filename)
        save_1d(multiple_layers, basin_name, ohc_1D, filename)
    compss_barrier()
    ellapsed = datetime.datetime.now() - start
    logger.info('Total ellapsed time: %s', ellapsed)

@task(returns=2)
def load_basins():
    regions = iris.load(REGIONS_FILE)
    basins = {}
    basin_name = []
    for region in regions:
        name = region.name()
        if name in ('nav_lat', 'nav_lon'):
            continue
        basin_name.append(name)
        extract_region = region.extract(iris.Constraint(z=1))
        basins[name] = extract_region.data.astype(np.float32)
    return basin_name, basins


@task(returns=1, filepath=FILE_IN)
def load_thetao(filepath):
    thetao = iris.load_cube(filepath, 'sea_water_potential_temperature')
    thetao_data = thetao.data.astype(np.float32)
    thetao_data += 273.15
    return thetao_data

@task()
def save_2d(layers, basins, ohc_2D, filename):
    ohc_cube = []
    output_folder = '/esarchive/scratch/Earth/jvegas/numba_outputs'
    for i, layer in enumerate(layers):
        ohc_cube.append(
            iris.cube.Cube(
                ohc_2D[i],
                long_name='Ocean heat content {0[0]} to {0[1]} '
                          'meters'.format(layer)
            )
        )
    iris.save(
        ohc_cube,
        '/esarchive/scratch/sloosvel/numba_outputs/ohc_{0}.nc'
        .format(device), zlib=True
    )

@task()
def save_1d(layers, basins, ohc_1D, filepath):
    ohc_cube = []
    output_folder = '/esarchive/scratch/Earth/jvegas/numba_outputs'
    filename = os.path.basename(filepath)
    for i, layer in enumerate(layers):
        ohc_1D = []
        for j, basin in enumerate(basins):
            ohc_1D.append(iris.cube.Cube(ohc1D[j][:],
                          long_name='{0}'.format(basin)))
        iris.save(
            ohc_1D,
            os.path.join(
                output_folder,
                filename.replace('thetao', 'ohc1D-{0[0]}-{0[1]}'.format(layer))
            ),
            zlib=True
        )
if __name__ == '__main__':
    main()
